/*
 *  TOPPERS/A-WDG
 *      Automotive Watchdog Driver
 *
 *  Copyright (C) 2014-2015 by Center for Embedded Computing Systems
 *                             Graduate School of Information Science, Nagoya Univ., JAPAN
 *  Copyright (C) 2014-2015 by AISIN COMCRUISE Co., Ltd., JAPAN
 *  Copyright (C) 2014-2015 by FUJI SOFT INCORPORATED, JAPAN
 *  Copyright (C) 2014-2015 by NEC Communication Systems, Ltd., JAPAN
 *  Copyright (C) 2014-2015 by Panasonic Advanced Technology Development Co., Ltd., JAPAN
 *  Copyright (C) 2014-2015 by SCSK Corporation, JAPAN
 *  Copyright (C) 2014-2015 by Sunny Giken Inc., JAPAN
 *  Copyright (C) 2014-2015 by TOSHIBA CORPORATION, JAPAN
 *  Copyright (C) 2014-2015 by Witz Corporation
 *
 *  上記著作権者は，以下の(1)〜(4)の条件を満たす場合に限り，本ソフトウェ
 *  ア（本ソフトウェアを改変したものを含む．以下同じ）を使用・複製・改
 *  変・再配布（以下，利用と呼ぶ）することを無償で許諾する．
 *  (1) 本ソフトウェアをソースコードの形で利用する場合には，上記の著作
 *      権表示，この利用条件および下記の無保証規定が，そのままの形でソー
 *      スコード中に含まれていること．
 *  (2) 本ソフトウェアを，ライブラリ形式など，他のソフトウェア開発に使
 *      用できる形で再配布する場合には，再配布に伴うドキュメント（利用
 *      者マニュアルなど）に，上記の著作権表示，この利用条件および下記
 *      の無保証規定を掲載すること．
 *  (3) 本ソフトウェアを，機器に組み込むなど，他のソフトウェア開発に使
 *      用できない形で再配布する場合には，次のいずれかの条件を満たすこ
 *      と．
 *    (a) 再配布に伴うドキュメント（利用者マニュアルなど）に，上記の著
 *        作権表示，この利用条件および下記の無保証規定を掲載すること．
 *    (b) 再配布の形態を，別に定める方法によって，TOPPERSプロジェクトに
 *        報告すること．
 *  (4) 本ソフトウェアの利用により直接的または間接的に生じるいかなる損
 *      害からも，上記著作権者およびTOPPERSプロジェクトを免責すること．
 *      また，本ソフトウェアのユーザまたはエンドユーザからのいかなる理
 *      由に基づく請求からも，上記著作権者およびTOPPERSプロジェクトを
 *      免責すること．
 *
 *  本ソフトウェアは，AUTOSAR（AUTomotive Open System ARchitecture）仕
 *  様に基づいている．上記の許諾は，AUTOSARの知的財産権を許諾するもので
 *  はない．AUTOSARは，AUTOSAR仕様に基づいたソフトウェアを商用目的で利
 *  用する者に対して，AUTOSARパートナーになることを求めている．
 *
 *  本ソフトウェアは，無保証で提供されているものである．上記著作権者お
 *  よびTOPPERSプロジェクトは，本ソフトウェアに関して，特定の使用目的
 *  に対する適合性も含めて，いかなる保証も行わない．また，本ソフトウェ
 *  アの利用により直接的または間接的に生じたいかなる損害に関しても，そ
 *  の責任を負わない．
 *
 *  $Id: Wdg.c 3492 2017-03-14 05:45:23Z suzuki-kawaguchi $
 */

/* Wdg.c */
#include "Wdg.h"
#include "Wdg_Target.h"
#include "Wdg_Cbk.h"
#include "SchM_Wdg.h"
#include "Det.h"
#include "Dem.h"	/* [WDG062][WDG080][WDG105] */
#include "MemMap.h"

/*
 *  内部関数のプロトタイプ宣言
 */
LOCAL_INLINE uint16 set_trigger_cnt(uint16 timeout, uint16 interrupt_period);

/*
 *  現在実行中のWdgコンフィギュレーション情報
 */
const Wdg_ConfigType *p_cur_wdg_config;

/*
 *  Wdgの初期状態
 */
#if WDG_DEV_ERROR_DETECT != FALSE
static uint8 wdg_state = WDG_UNINIT;
#endif /* WDG_DEV_ERROR_DETECT != FALSE */

/*
 *  [WDG106] Wdg_Init
 */
void
Wdg_Init(const Wdg_ConfigType *ConfigPtr)
{
	TRIGGER_CB			*p_trigger_cb;
	const WDG_MODE_INIB	*p_current_mode;

	SchM_Enter_Wdg_Reentrant();

	/* [WDG089] 引数ConfigPtrがNULL_PTRである場合 */
	WDG_CHECK_PARAM((ConfigPtr != NULL_PTR), WdgServiceId_Init, WDG_E_PARAM_POINTER);

	/* [NWDG004] Wdgのモジュール状態がWDG_BUSYの場合 */
	WDG_CHECK_PARAM((wdg_state != WDG_BUSY), WdgServiceId_Init, WDG_E_DRIVER_STATE);

	/* 現在実行中のコンフィグレーション情報を保持 */
	p_cur_wdg_config = ConfigPtr;

	p_trigger_cb = p_cur_wdg_config->p_trigger_cb;
	p_trigger_cb->current_mode = p_cur_wdg_config->WdgDefaultMode;

	/* [WDG001] ハードウェア初期化*/
	if (p_trigger_cb->current_mode != WDGIF_OFF_MODE) {
		/* Wdgモード初期化ブロック取得 */
		p_current_mode = get_wdg_mode_inib(p_trigger_cb->current_mode);

		/* [NWDG006] p_current_modeがNULL_PTRの場合 */
		WDG_CHECK_PARAM((p_current_mode != NULL_PTR), WdgServiceId_Init, WDG_E_PARAM_POINTER);

		/* トリガ数設定 */
		p_trigger_cb->trigger_cnt = set_trigger_cnt(WDG_INIT_TIMEOUT, p_current_mode->interrupt_period);

		/* [WDG051][WDG101] 依存部開始 */
		wdg_target_start(p_current_mode);
	}
	else {
		/* トリガ数設定 */
		p_trigger_cb->trigger_cnt = 0U;

		/* 依存部停止 */
		wdg_target_off_mode();
	}

	/* [WDG019][WDG100] 状態を設定 */
	WDG_SET_STATE(WDG_IDLE);

  error_exit:
	SchM_Exit_Wdg_Reentrant();
	return;
}

/*
 *  [WDG107] Wdg_SetMode
 */
Std_ReturnType
Wdg_SetMode(WdgIf_ModeType WdgMode)
{
	Std_ReturnType		ercd;

#if WDG_SET_MODE_VALID == TRUE
	uint16				timeout;
	WdgIf_ModeType		pre_wdgmode;
	TRIGGER_CB			*p_trigger_cb;
	const WDG_MODE_INIB	*p_pre_mode_inib;
	const WDG_MODE_INIB	*p_current_mode;

	/* [NWDG005] 排他しない */

	/* [WDG017][WDG035] Wdgのモジュール状態がWDG_IDLEでない場合 */
	WDG_CHECK_INITIALIZED_ERCD(WdgServiceId_SetMode);

	/* [WDG091] 引数WdgModeが範囲外である場合 */
	WDG_CHECK_PARAM_ERCD(WdgMode <= WDGIF_FAST_MODE, WdgServiceId_SetMode, WDG_E_PARAM_MODE);

#if WDG_DISABLE_ALLOWED == FALSE
	/* [WDG026] 引数WdgModeがWDGIF_OFF_MODEである場合 */
	WDG_CHECK_DEM_EXIT_ERCD(WdgMode != WDGIF_OFF_MODE, WDG_DEM_DISABLE_REJECTED);
#endif /* WDG_DISABLE_ALLOWED == FALSE */

	/* [WDG018][WDG052] Wdgのモジュール状態をWDG_BUSYに設定する */
	WDG_SET_STATE(WDG_BUSY);

	p_trigger_cb = p_cur_wdg_config->p_trigger_cb;

	if (p_trigger_cb->current_mode != WdgMode) {
		/* 前回のモード保持 */
		pre_wdgmode = p_trigger_cb->current_mode;

		/* [WDG160] 現在のモード更新 */
		p_trigger_cb->current_mode = WdgMode;

		if (p_trigger_cb->current_mode != WDGIF_OFF_MODE) {
			/* Wdgモード初期化ブロック取得 */
			p_current_mode = get_wdg_mode_inib(p_trigger_cb->current_mode);

			/* [NWDG006] p_current_modeがNULL_PTRの場合 */
			WDG_CHECK_PARAM_ERCD((p_current_mode != NULL_PTR), WdgServiceId_SetMode, WDG_E_PARAM_POINTER);

			/* [WDG145] 割り込み周期が異なる場合トリガ数再設定 */
			if (pre_wdgmode != WDGIF_OFF_MODE) {
				/* 前回のWdgモード初期化ブロック取得 */
				p_pre_mode_inib = get_wdg_mode_inib(pre_wdgmode);

				/* [NWDG006] p_pre_mode_inibがNULL_PTRの場合 */
				WDG_CHECK_PARAM_ERCD((p_pre_mode_inib != NULL_PTR), WdgServiceId_SetMode, WDG_E_PARAM_POINTER);
				if (p_pre_mode_inib->interrupt_period != p_current_mode->interrupt_period) {
					timeout = wdg_target_reset_timeout(p_trigger_cb->trigger_cnt, p_pre_mode_inib->interrupt_period);

					p_trigger_cb->trigger_cnt = set_trigger_cnt(timeout, p_current_mode->interrupt_period);
				}
			}
			else {
				/* トリガ数再設定 */
				p_trigger_cb->trigger_cnt = 0U;

				/* ウォッチドッグタイマ起動処理 */
				wdg_timer_init(p_current_mode);
			}

			/* 依存部モード設定 */
			wdg_target_set_mode(p_current_mode);
		}
		else {
			/* トリガ数再設定 */
			p_trigger_cb->trigger_cnt = 0U;

			/* 依存部停止 */
			wdg_target_off_mode();
		}
	}

	/* [WDG018] Wdgのモジュール状態をWDG_IDLEに設定する */
	WDG_SET_STATE(WDG_IDLE);

	/* [WDG103] モード切替え成功 */
	ercd = E_OK;
#else /* WDG_SET_MODE_VALID != TRUE */
	/* [NWDG007] モード切り替えが無効である場合 */
	WDG_CHECK_PARAM_ERCD(WDG_SET_MODE_VALID != FALSE, WdgServiceId_SetMode, WDG_E_PARAM_MODE);
#endif /* WDG_SET_MODE_VALID == TRUE */

  error_exit:
	/* [WDG016] モード切換えが失敗した場合 */
	WDG_CHECK_DEM_ERCD(ercd != E_NOT_OK, WDG_DEM_MODE_FAILED);

	return(ercd);
}

/*
 *  [WDG155] Wdg_SetTriggerCondition
 */
void
Wdg_SetTriggerCondition(uint16 timeout)
{
	TRIGGER_CB			*p_trigger_cb;
	const WDG_MODE_INIB	*p_current_mode;

	/* [NWDG005] 排他しない */

	/* [WDG146] 引数timeoutが範囲外である場合 */
	WDG_CHECK_PARAM(timeout <= WDG_MAX_TIMEOUT, WdgServiceId_SetTriggerCondition, WDG_E_PARAM_TIMEOUT);

	/* Wdgのモジュール状態をWDG_BUSYに設定する */
	WDG_SET_STATE(WDG_BUSY);

	p_trigger_cb = p_cur_wdg_config->p_trigger_cb;

	if (p_trigger_cb->current_mode != WDGIF_OFF_MODE) {

		/* Wdgモード初期化ブロック取得 */
		p_current_mode = get_wdg_mode_inib(p_trigger_cb->current_mode);

		/* [NWDG006] p_current_modeがNULL_PTRの場合 */
		WDG_CHECK_PARAM((p_current_mode != NULL_PTR), WdgServiceId_Init, WDG_E_PARAM_POINTER);

		/* [WDG136] トリガ数設定 */
		p_trigger_cb->trigger_cnt = set_trigger_cnt(timeout, p_current_mode->interrupt_period);
	}

	/* Wdgのモジュール状態をWDG_IDLEに設定する */
	WDG_SET_STATE(WDG_IDLE);

  error_exit:
	return;
}

/*
 *  [WDG109][WDG099] Wdg_GetVersionInfo
 */
/* [WDG068] WDG_VERSION_INFO_API */
#if WDG_VERSION_INFO_API != FALSE
void
Wdg_GetVersionInfo(Std_VersionInfoType *versioninfo)
{
	/* [WDG174] 引数versioninfoがNULL_PTRである場合 */
	WDG_CHECK_PARAM(versioninfo != NULL_PTR, WdgServiceId_GetVersionInfo, WDG_E_PARAM_POINTER);

	/* [WDG067]バージョン情報を取得する */
	versioninfo->vendorID = WDG_VENDOR_ID;
	versioninfo->moduleID = WDG_MODULE_ID;
	versioninfo->sw_major_version = WDG_SW_MAJOR_VERSION;
	versioninfo->sw_minor_version = WDG_SW_MINOR_VERSION;
	versioninfo->sw_patch_version = WDG_SW_PATCH_VERSION;

  error_exit:
	return;
}
#endif /* WDG_VERSION_INFO_API != FALSE */

/*
 *  トリガ数設定
 */
LOCAL_INLINE uint16
set_trigger_cnt(uint16 timeout, uint16 interrupt_period)
{
	uint16 ret_trigger_cnt;

	/* [WDG140] 引数のタイムアウト値が0の場合 */
	if (timeout == 0U) {
		ret_trigger_cnt = 0U;
	}
	/* [WDG138][WDG139][DWDG001] トリガ数設定 */
	else {
		ret_trigger_cnt = ((timeout + interrupt_period - 1) / interrupt_period) + 1U;
	}

	return(ret_trigger_cnt);
}

/*
 *  以下，未サポート機能
 */

/* <WDG163> */
void
Wdg_Cbk_GptNotification(void)
{
	return;
}
